@@ -19,6 +19,7 @@
 #include "key.h"
 #include "keynode.h"
 #include "rsc.h"
+#include "names.h"
 #include "sbuffer.h"
 #include "tex_read.h"
 #include "wordlist.h"
@@ -41,20 +42,25 @@
  static int eval__fmt _ARG((KeyNode kn,Record rec));/* key.c                 */
  static int eval_fmt _ARG((KeyNode kn,Record rec));/* key.c                  */
  static int fmt__parse _ARG((char **sp,KeyNode *knp));/* key.c               */
+ static int fmt_c_names _ARG((char *line,int min,int max,int not));/* key.c  */
+ static int fmt_c_string _ARG((char * s,int min,int max,int not));/* key.c   */
+ static int fmt_c_words _ARG((char *line,int min,int max,int not,int ignore));/* key.c*/
  static int fmt_parse _ARG((char **sp,KeyNode *knp));/* key.c                */
  static void eval__special _ARG((KeyNode kn,Record rec));/* key.c            */
- static void fmt_1_name _ARG((char **word,size_t len,int maxlen,char *trans,int initialp));/* key.c*/
  static void fmt_digits _ARG((char *s,int n));	   /* key.c                  */
- static void fmt_names _ARG((char *line,int maxname,int maxlen,char *trans,int initialp));/* key.c*/
+ static void fmt_names _ARG((char *line,int maxname,int post,char *trans));/* key.c*/
+ static void fmt_string _ARG((char * s,int n,char *trans));/* key.c          */
  static void fmt_title _ARG((char *line,int len,int in,char *trans,int ignore,char *sep));/* key.c*/
- static void key_copy _ARG((char * s,int n,char *trans));/* key.c            */
  static void key_init _ARG((void));		   /* key.c                  */
+ static void push_S _ARG((unsigned char *s,unsigned char *trans));/* key.c   */
  static void push_s _ARG((unsigned char *s,int max,unsigned char *trans));/* key.c*/
  void Push_Word _ARG((char *s));		   /* key.c                  */
  void add_format _ARG((char *s));		   /* key.c                  */
  void add_ignored_word _ARG((char *s));		   /* key.c                  */
  void add_sort_format _ARG((char *s));		   /* key.c                  */
+ void def_format_type _ARG((char *s));		   /* key.c                  */
  void free_key_node _ARG((KeyNode kn));		   /* key.c                  */
+ void init_key _ARG((int state));		   /* key.c                  */
  void make_key _ARG((Record rec));		   /* key.c                  */
  void make_sort_key _ARG((Record rec));		   /* key.c                  */
  void mark_key _ARG((Record rec));		   /* key.c                  */
@@ -86,12 +92,31 @@
 #define DetexUpper 2
 
  static StringBuffer *key_sb = (StringBuffer*)NULL;
+ static StringBuffer *tmp_sb = (StringBuffer*)NULL;
 
 /*---------------------------------------------------------------------------*/
 
  static KeyNode key_tree      = (KeyNode)0;
  static KeyNode sort_key_tree = (KeyNode)0;
 
+
+ static NameNode format[NUMBER_OF_FORMATS];
+
+
+#define SkipSpaces(CP)	  while ( is_space(*CP) ) ++(CP)
+#define SkipAllowed(CP)	  while ( is_allowed(*CP) ) ++(CP)
+#define ParseNumber(CP,N) N = 0;	\
+			  while(is_digit(*CP) ){ N = N*10 + (*CP)-'0'; ++CP; }
+#define Expect(CP,C,RET)  SkipSpaces(CP);				\
+			  if ( *CP == C ) { ++(CP); }			\
+			  else { ErrPrintF("*** BibTool: Missing %c",C); \
+				   return RET; }
+#define MakeNode(KNP,TYPE,SP,CP) c = *CP; *CP = '\0';			\
+			  *KNP = new_key_node(TYPE,symbol(*SP));	\
+			  *CP  = c;
+#define ParseOrReturn(CP,NODEP)						\
+	if ( (ret=fmt_parse(CP,NODEP)) != 0  ) return ret
+
 
 /*****************************************************************************/
 /***				Private word stack			   ***/
@@ -277,29 +302,72 @@ static char * itostr(i,digits)			   /*			     */
 /***			       Key Init Section				   ***/
 /*****************************************************************************/
 
+/*-----------------------------------------------------------------------------
+** Function:	init_key()
+** Purpose:	Global initializations for the key module.
+**		
+**
+** Arguments:	none
+** Returns:	nothing
+**___________________________________________________			     */
+void init_key(state)				   /*                        */
+  int  state;
+{ int  i;					   /*			     */
+  char *s;
+  						   /*                        */
+  switch (state)
+  { case 0:
+      for (i=0;i<NUMBER_OF_FORMATS;i++)		   /*                        */
+      { format[i] = NameNULL;			   /*                        */
+      }						   /*                        */
+      break;
+    case 1:
+      if ( format[0] == NameNULL )		   /*                        */
+      { s = new_string("%l");
+	format[0] = name_format(s);
+	free(s);
+	NameMid(format[0]) = InterNameSep;
+      }
+      
+      if ( format[1] == NameNULL )		   /*                        */
+      { s = new_string("%l%1f");
+	format[1] = name_format(s);
+	free(s);
+	NameMid(format[1]) = InterNameSep;
+      }
+      if (NextName(format[1]))
+      { NamePre(NextName(format[1])) = NamePreSep;		
+      }
+      break;
+  }
+}						   /*------------------------*/
+
+
 /*-----------------------------------------------------------------------------
 ** Function:	key_init()
 ** Purpose:	Perform initializations for key generation.
 **		The string buffer is opened.
-**		Ignored words are initialized.
+**		Ignored words are initialized if necessary.
 ** Arguments:   none
 ** Returns:	nothing
 **___________________________________________________			     */
 static void key_init()				   /*			     */
-{						   /*			     */
+{ char * s;					   /* Intermediate string ptr*/
 #ifdef INITIALIZE_IGNORED_WORDS
   static char *word_list[] =			   /* default ignored words. */
   { INITIALIZE_IGNORED_WORDS, NULL };		   /* Mark the end with NULL */
   register char**wp;				   /*			     */
 #endif
 						   /*			     */
-  if ( key_sb == (StringBuffer*)0 )		   /*			     */
-  { if ( (key_sb=sbopen()) == (StringBuffer*)0 )   /* open string buffer     */
+  if ( key_sb == (StringBuffer*)0 )		   /* Is it the first time?  */
+  {						   /*                        */
+    if ( (key_sb=sbopen()) == (StringBuffer*)0 )   /* open string buffer     */
     { OUT_OF_MEMORY("key generation."); }	   /*			     */
 #ifdef INITIALIZE_IGNORED_WORDS
     for ( wp=word_list; *wp!=NULL; ++wp )	   /* add ignored words.     */
     { add_ignored_word(*wp); }			   /*			     */
 #endif
+ 						   /*                        */
   }						   /*			     */
 }						   /*------------------------*/
 
@@ -508,10 +576,9 @@ static void fmt_title(line,len,in,trans,ignore,sep)/*			     */
 { int	 nw, i, j;				   /*			     */
   char	 *s;					   /*			     */
   int	 first = TRUE;				   /*			     */
-  static StringBuffer* sb = (StringBuffer*)0;	   /*			     */
 						   /*			     */
-  if (	 sb == (StringBuffer*)0			   /*			     */
-      && (sb=sbopen()) == (StringBuffer*)0 )	   /*			     */
+  if (	 tmp_sb == (StringBuffer*)0		   /*			     */
+      && (tmp_sb=sbopen()) == (StringBuffer*)0 )   /*			     */
   { OUT_OF_MEMORY("fmt_title()"); } 		   /*			     */
 						   /*			     */
   ResetWords;					   /*                        */
@@ -519,10 +586,10 @@ static void fmt_title(line,len,in,trans,ignore,sep)/*			     */
  						   /*                        */
   for ( i=0; i < nw; ++i )		   	   /*			     */
   {						   /*			     */
-    sbrewind(sb);				   /* Reset the string buffer*/
+    sbrewind(tmp_sb);				   /* Reset the string buffer*/
     for ( s=words[i]; *s; ++s )		   	   /* Translate the current  */
-    { (void)sbputchar(trans[*s],sb); }	   	   /*  word into the sbuffer */
-    s = sbflush(sb);				   /* Get the translated word*/
+    { (void)sbputchar(trans[*s],tmp_sb); }	   /*  word into the sbuffer */
+    s = sbflush(tmp_sb);			   /* Get the translated word*/
 						   /*			     */
     if ( ! ignore || 				   /*                        */
 	 ! find_word(s,ignored_words[(*s)&31] ) )  /*			     */
@@ -540,11 +607,90 @@ static void fmt_title(line,len,in,trans,ignore,sep)/*			     */
   }						   /*			     */
 }						   /*------------------------*/
 
+/*-----------------------------------------------------------------------------
+** Function:	fmt_c_words()
+** Purpose:	Count a list of words.
+**		
+**
+** Arguments:
+**	line	string to analyze
+**	min	minimum
+**	max	maximum or 0
+**	not	negation flag
+**	ignore	flag to indicate if certain words should be ignored.
+** Returns:	
+**___________________________________________________			     */
+static int fmt_c_words(line,min,max,not,ignore)	   /*			     */
+  char	 *line;					   /*			     */
+  int	 min;					   /*			     */
+  int	 max;					   /*			     */
+  int	 not;					   /*			     */
+  int	 ignore;				   /*			     */
+{ int	 n, i, nw;				   /*			     */
+						   /*			     */
+  ResetWords;					   /*                        */
+  nw = deTeX(*line=='{'?line+1:line,push_word,FALSE);/*			     */
+  n = 0;					   /*                        */
+ 						   /*                        */
+  for ( i=0; i < nw; ++i )		   	   /*			     */
+  {						   /*			     */
+    if ( !ignore || 				   /*                        */
+	 !find_word(words[i],ignored_words[(*words[i])&31]) )/*		     */
+    { n++; }					   /*			     */
+  }						   /*			     */
+ 						   /*                        */
+  if ( n < min || (max>0 && n> max))		   /*                        */
+  { return (not?FALSE:TRUE); }			   /*                        */
+ 						   /*                        */
+  return (not?TRUE:FALSE);			   /*                        */
+}						   /*------------------------*/
+
 
 /*****************************************************************************/
 /***		         Name Formatting Section			   ***/
 /*****************************************************************************/
 
+
+/*-----------------------------------------------------------------------------
+** Function:	def_format_type()
+** Purpose:	
+**		
+**
+** Arguments:
+**	s
+** Returns:	nothing
+**___________________________________________________			     */
+void def_format_type(s)				   /*                        */
+  char *s;					   /*                        */
+{ int  n;					   /*                        */
+  char *cp;					   /*                        */
+  char c;					   /*                        */
+ 						   /*                        */
+  SkipSpaces(s);				   /*                        */
+  n = 0;					   /*                        */
+  while ( is_digit(*s) ) { n = n*10 + (*s++) - '0'; }/*                      */
+  if ( n >= NUMBER_OF_FORMATS )			   /*                        */
+  { WARNING("Format type number is out of range.");/*                        */
+    return;					   /*                        */
+  }						   /*                        */
+  if ( n == 0 )			   		   /*                        */
+  { VerbosePrint1("Name format specifier 0 has been changed.\n");/*          */
+  }						   /*                        */
+  else if ( n == 1 )				   /*                        */
+  { VerbosePrint1("Name format specifier 1 has been changed.\n");/*          */
+  }						   /*                        */
+ 						   /*                        */
+  SkipSpaces(s);				   /*                        */
+  if ( *s == '=' ) s++;				   /*                        */
+  Expect(s,'"',);				   /*                        */
+  cp = s;					   /*                        */
+  while ( *cp && *cp != '"' ) cp++;		   /*                        */
+  c   = *cp;					   /*                        */
+  *cp = '\0';					   /*                        */
+  format[n] = name_format(s);			   /*                        */
+  *cp = c;					   /*                        */
+}						   /*------------------------*/
+
 /*-----------------------------------------------------------------------------
 ** Function:	fmt_names()
 ** Purpose:	Format a list of names separated by 'and'.
@@ -554,114 +700,84 @@ static void fmt_title(line,len,in,trans,ignore,sep)/*			     */
 **	maxname	Number of names to consider
 **	maxlen  Number of characters per name to consider
 **	trans	Translation table
-**	initialp Use initials?
 ** Returns:	nothing
 **___________________________________________________			     */
-static void fmt_names(line,maxname,maxlen,trans,initialp)/*		     */
-  char	*line;					   /* Name list string	     */
-  int	maxname;				   /* number of names b4 etal*/
-  int   maxlen;					   /* number of relevant char*/
-  char	*trans;					   /* Translation table	     */
-  int	initialp;				   /* if TRUE use initials   */
-{ int	wp,					   /*			     */
-	i,					   /*			     */
-	beg;					   /*			     */
-						   /*			     */
+static void fmt_names(line,maxname,post,trans)	   /*		             */
+  char	      *line;				   /* Name list string	     */
+  int	      maxname;				   /* number of names b4 etal*/
+  int         post;				   /* number of relevant char*/
+  char	      *trans;				   /* Translation table	     */
+{ int	      wp,				   /*			     */
+	      i;				   /*			     */
+  char	      *s;				   /*                        */
+  static char *and   = "&";			   /*                        */
+  static char *comma = ",";			   /*                        */
+  static int  undef_warning = FALSE;		   /*                        */
+  						   /*                        */
+  if (   post < 0				   /*                        */
+      || post >= NUMBER_OF_FORMATS		   /*                        */
+      || format[post] == NameNULL )		   /*                        */
+  { if (undef_warning)				   /*                        */
+    { ErrPrintF("*** BibTool: Format %d is not defined. Ignored.\n",/*       */
+		post);				   /*                        */
+      undef_warning = TRUE;			   /*                        */
+    }						   /*                        */
+    return;					   /*                        */
+  }						   /*                        */
+ 						   /*                        */
   ResetWords;					   /*                        */
   wp = deTeX(*line=='{'?line+1:line,push_word,TRUE);/*			     */
+  words[wp] = NULL;				   /*                        */
  						   /*                        */
-  for ( beg=0; beg<wp; beg=i+1)			   /*			     */
-  {						   /*			     */
-    if ( maxname-- <= 0 ) { PushS(EtAl); return; } /*			     */
-						   /*			     */
-    for ( i=beg;				   /* Search the next 'and'  */
-	  i<wp && strcmp(words[i],"and") != 0;	   /*			     */
-	  ++i )	{ }				   /*			     */
-    switch ( i-beg )				   /*			     */
-    { case 0: break;				   /*			     */
-      case 1:					   /*			     */
-	if ( strcmp(words[beg],"others") == 0 )	   /*			     */
-	{ PushS(EtAl); return; }		   /*			     */
-      default:					   /*			     */
-	if ( beg>0 ) PushS(NameNameSep);	   /*			     */
-						   /*			     */
-	fmt_1_name(&words[beg],i-beg,maxlen,trans,initialp);/*		     */
-    }						   /*			     */
-  }						   /*			     */
+  for ( i=0; i<wp; i++)			   	   /*			     */
+  { if ( strcmp(words[i],"and") == 0 )		   /*                        */
+    { words[i] = and; }				   /*                        */
+    else if ( strcmp(words[i],",") == 0 )	   /*                        */
+    { words[i] = comma; }			   /*                        */
+  }						   /*                        */
+ 						   /*                        */
+  PushS(pp_list_of_names(words,		   	   /*                        */
+			 format[post],		   /*                        */
+			 trans,			   /*                        */
+			 maxname,		   /*                        */
+			 comma,		   	   /*                        */
+			 and,		   	   /*                        */
+			 NameNameSep,EtAl));	   /*                        */
 }						   /*------------------------*/
 
 /*-----------------------------------------------------------------------------
-** Function:	fmt_1_name()
-** Purpose:	Format a single name.
+** Function:	fmt_c_names()
+** Purpose:	Count a list of names
+**		
+**
 ** Arguments:
-**	word	array of constituents of the name
-**	len	
-**	maxlen  Number of characters per name to consider
-**	trans	Translation table
-**	initialp
+**	line	string to analyze
+**	min	minimum
+**	max	maximum or 0
+**	not	negation flag
 ** Returns:	nothing
 **___________________________________________________			     */
-static void fmt_1_name(word,len,maxlen,trans,initialp)/*		     */
-  char	      **word;				   /*			     */
-  size_t      len;				   /*			     */
-  int         maxlen;				   /*                        */
-  char	      *trans;				   /* Translation table	     */
-  int	      initialp;				   /*			     */
-{ int	      i;				   /*			     */
-  int	      commas = 0;			   /*			     */
-  char	      *type;				   /*			     */
-  char	      buffer[2];			   /*			     */
-						   /*			     */
-  if ( (type=malloc(len+1)) == NULL )		   /*                        */
-  { OUT_OF_MEMORY("fmt_1_name()"); }  		   /*			     */
+static int fmt_c_names(line,min,max,not)	   /*		             */
+  char	      *line;				   /* Name list string	     */
+  int         min;				   /* number of relevant char*/
+  int	      max;				   /* number of names b4 etal*/
+  int         not;				   /* negation flag          */
+{ int	      wp,				   /*			     */
+	      i,				   /*                        */
+	      n;				   /*			     */
+  						   /*                        */
+  ResetWords;					   /*                        */
+  wp = deTeX(*line=='{'?line+1:line,push_word,TRUE);/*			     */
+  words[wp] = NULL;				   /*                        */
  						   /*                        */
-  for ( i=0;i<len;++i )				   /*			     */
-  { if ( *word[i] == ',' )			   /*			     */
-    { ++commas; type[i] = ','; }		   /*			     */
-    else if ( case_cmp(word[i],"jr") )		   /*			     */
-    { type[i] = 'j'; }				   /*			     */
-    else type[i] = 'f';				   /*			     */
-  }						   /*			     */
-						   /*			     */
-  switch ( commas )				   /*			     */
-  { case 0: type[len-1] = 'n'; break;		   /* ff nn		     */
-    case 1:					   /*			     */
-      if ( type[len-1] == 'j' && len >= 2 )	   /* ff nn, jj		     */
-      { type[len-3] = 'n'; }			   /*			     */
-      else					   /* nn, ff		     */
-      { for (i=0; i<len&&type[i]!=','; ++i)	   /*			     */
-	  type[i] = 'n';			   /*			     */
-      }						   /*			     */
-      break;					   /*			     */
-    default:					   /*			     */
-      for (i=0; i<len&&type[i]!=','; ++i)	   /*			     */
-	type[i] = 'n';				   /* nn, jj, ff	     */
-      break;					   /*			     */
-  }						   /*			     */
-						   /*			     */
-  commas = 1;					   /* comma is used to	     */
-  for ( i=0;i<len;++i )				   /*  indicate the beginning*/
-  { if ( type[i] == 'n' )			   /*			     */
-    { if ( commas ) { commas = 0; }		   /*			     */
-      else { PushS(NameNameSep); }		   /*			     */
-      PushStr(word[i],maxlen);			   /*			     */
-    }						   /*			     */
-  }						   /*			     */
-						   /*			     */
-  if ( initialp )				   /*			     */
-  {						   /*			     */
-    commas = 1;					   /* comma is used to	     */
-    for ( i=0;i<len;++i )			   /*  indicate the beginning*/
-    { if ( type[i] == 'f' )			   /*			     */
-      { if ( commas )				   /*			     */
-	{ commas = 0; PushS(NamePreSep); }	   /*			     */
-	buffer[0] = *word[i];			   /*			     */
-	buffer[1] = '\0';			   /*			     */
-	PushS(buffer);				   /*			     */
-      }						   /*			     */
-    }						   /*			     */
-  }						   /*			     */
-  free(type);					   /*			     */
+  for ( i=0, n=1; i<wp; i++)			   /*			     */
+  { if ( strcmp(words[i],"and") == 0 ) { n++; }	   /*                        */
+  }						   /*                        */
+ 						   /*                        */
+  if ( n < min || (max>0 && n> max))		   /*                        */
+  { return (not?FALSE:TRUE); }			   /*                        */
+ 						   /*                        */
+  return (not?TRUE:FALSE);			   /*                        */
 }						   /*------------------------*/
 
 
@@ -698,7 +814,7 @@ static void fmt_digits(s,n)			   /*			     */
 /*****************************************************************************/
 
 /*-----------------------------------------------------------------------------
-** Function:	key_copy()
+** Function:	fmt_string()
 ** Purpose:	Push characters from s onto the string buffer.
 **		At most n characters are transfered.
 **		Letters are translated accoring to mode.
@@ -708,7 +824,7 @@ static void fmt_digits(s,n)			   /*			     */
 **	trans	Translation table
 ** Returns:	nothing
 **___________________________________________________			     */
-static void key_copy(s,n,trans)			   /*			     */
+static void fmt_string(s,n,trans)		   /*			     */
   register char * s;				   /*			     */
   register int	n;				   /*			     */
   register char *trans;				   /*			     */
@@ -725,6 +841,40 @@ static void key_copy(s,n,trans)			   /*			     */
 						   /*			     */
 }						   /*------------------------*/
 
+/*-----------------------------------------------------------------------------
+** Function:	fmt_c_string()
+** Purpose:	Count the number of characters in a string and compare it
+**		against an interval.
+**
+** Arguments:
+**	s	string to analyze
+**	min	minimum
+**	max	maximum or 0
+**	not	negation flag
+** Returns:	
+**___________________________________________________			     */
+static int fmt_c_string(s,min,max,not)		   /*			     */
+  register char * s;				   /*			     */
+  register int  min;				   /*			     */
+  register int  max;				   /*			     */
+  register int  not;				   /*			     */
+{ int           n = 0;				   /*			     */
+ 						   /*                        */
+  while ( *s )				   	   /*			     */
+  { if ( is_allowed(*s) ) { n++; }	   	   /*			     */
+    else if ( is_space(*s) )			   /*			     */
+    { n += strlen(TitleTitleSep);	   	   /*                        */
+      while ( is_space(*s) ) s++;		   /* skip over multiple SPC */
+    }   					   /*			     */
+    ++s;					   /*			     */
+  }						   /*			     */
+ 						   /*                        */
+  if ( n < min || (max>0 && n> max))		   /*                        */
+  { return (not?FALSE:TRUE); }			   /*                        */
+ 						   /*                        */
+  return (not?TRUE:FALSE);			   /*                        */
+}						   /*------------------------*/
+
 /*-----------------------------------------------------------------------------
 ** Function:	get_field()
 ** Purpose:	Evaluate the record rec.
@@ -740,7 +890,8 @@ static void key_copy(s,n,trans)			   /*			     */
 char *get_field(rec,name)		   	   /*			     */
   register Record rec;				   /*			     */
   register char	  *name;			   /*			     */
-{						   /*			     */
+{ static char     *crossref = NULL;		   /*			     */
+ 						   /*                        */
   if ( *name == '@' )				   /*			     */
   { if ( case_cmp(name+1,EntryName(RecordToken(rec))) )/*		     */
       return EntryName(RecordToken(rec));	   /*		             */
@@ -780,7 +931,20 @@ char *get_field(rec,name)		   	   /*			     */
       }						   /*                        */
       else { cpp++; cpp++; }			   /*			     */
     }						   /*			     */
+ 
+    if ( crossref==NULL ) crossref = symbol("crossref");
+ 
+    for ( cpp=RecordHeap(rec), n=RecordFree(rec);  /*			     */
+	  n > 0 && crossref != *cpp;		   /*			     */
+	  n -= 2 )				   /*			     */
+    { cpp++; cpp++; }			   /*			     */
+
+    if ( n > 0 && crossref == *cpp )
+    { cpp++;
+      printf("I should try to consult entry %s\nBut this isn't implemented yet",*cpp);
+    }
   }						   /*			     */
+ 						   /*                        */
   return (char*)0;				   /*			     */
 }						   /*------------------------*/
 
@@ -1027,21 +1191,6 @@ void add_sort_format(s)				   /*			     */
   (void)add_fmt_tree(s,&sort_key_tree);		   /*			     */
 }						   /*------------------------*/
 
-
-#define SkipSpaces(CP)	  while ( is_space(*CP) ) ++(CP)
-#define SkipAllowed(CP)	  while ( is_allowed(*CP) ) ++(CP)
-#define ParseNumber(CP,N) N = 0;	\
-			  while(is_digit(*CP) ){ N = N*10 + (*CP)-'0'; ++CP; }
-#define Expect(CP,C)	  SkipSpaces(CP);				\
-			  if ( *CP == C ) { ++(CP); }			\
-			  else { ErrPrintF("*** BibTool: Missing %c",C); \
-				   return(1); }
-#define MakeNode(KNP,TYPE,SP,CP) c = *CP; *CP = '\0';			\
-			  *KNP = new_key_node(TYPE,symbol(*SP));	\
-			  *CP  = c;
-#define ParseOrReturn(CP,NODEP)						\
-	if ( (ret=fmt_parse(CP,NODEP)) != 0  ) return ret
-
 /*-----------------------------------------------------------------------------
 ** Function:	fmt_parse()
 ** Purpose:	Parse the format specification given in the string *sp.
@@ -1088,7 +1237,7 @@ static int fmt__parse(sp,knp)			   /*			     */
 { register char *cp;				   /*			     */
   char		c;				   /*			     */
   int		ret;				   /*			     */
-  static char	*percent_chars = "lNntTdswW";	   /*                        */
+  static char	*percent_chars = "NntTdsSwWp";     /*                        */
 						   /*			     */
   cp = *sp;					   /*			     */
 						   /*			     */
@@ -1104,14 +1253,19 @@ static int fmt__parse(sp,knp)			   /*			     */
 	{ int  pre  = -1,			   /*			     */
 	       post = -1,			   /* %[-][0-9]*[.[0-9]*[a-Z]*/
 	       type = 0;			   /*	([a-Z]*)	     */
-						   /*			     */
 	  switch ( *++cp )			   /*			     */
 	  { case '+': type = NodePlusMask;  ++cp; break;/*		     */
 	    case '-': type = NodeMinusMask; ++cp; break;/*		     */
 	  }					   /*			     */
 	  ParseNumber(cp,pre);			   /*			     */
-	  if ( *cp == '.' )			   /*			     */
-	  { ++cp; ParseNumber(cp,post); }	   /*			     */
+	  if ( *cp == '.' || *cp == ',' || *cp == '-' )/*		     */
+	  { cp++;				   /*                        */
+	    ParseNumber(cp,post);		   /*                        */
+	  }	   				   /*			     */
+	  if ( *cp == '#' )			   /*                        */
+	  { cp++;				   /*                        */
+	    type |= NodeCountMask;		   /*                        */
+	  }					   /*                        */
 	  if ( !is_alpha(*cp) )			   /*			     */
 	  { ErrPrintF("*** BibTool: Missing format type before: %s",cp);/*   */
 	    *sp = cp; return(1);		   /*			     */
@@ -1121,7 +1275,7 @@ static int fmt__parse(sp,knp)			   /*			     */
 	    *sp = cp; return 1;			   /*			     */
 	  }					   /*			     */
 	  type |= *(cp++);			   /*			     */
-	  Expect(cp,'(');			   /*			     */
+	  Expect(cp,'(',1);			   /*			     */
 	  SkipSpaces(cp);			   /*			     */
 	  *sp = cp;				   /*			     */
 	  SkipAllowed(cp);			   /*			     */
@@ -1129,7 +1283,7 @@ static int fmt__parse(sp,knp)			   /*			     */
 	  NodePre(*knp)	 = pre;			   /*			     */
 	  NodePost(*knp) = post;		   /*			     */
 	  SkipSpaces(cp);			   /*			     */
-	  Expect(cp,')');			   /*			     */
+	  Expect(cp,')',1);			   /*			     */
 	  *sp = cp;				   /*			     */
 	  knp = & NodeNext(*knp);		   /*			     */
 	}					   /*			     */
@@ -1140,14 +1294,14 @@ static int fmt__parse(sp,knp)			   /*			     */
 	*sp = cp;				   /*			     */
 	SkipAllowed(cp);			   /*			     */
 	MakeNode(knp,NodeTEST,sp,cp)		   /*			     */
-	Expect(cp,')');				   /*			     */
-	Expect(cp,'{');				   /*			     */
+	Expect(cp,')',1);			   /*			     */
+	Expect(cp,'{',1);			   /*			     */
 	*sp = cp;				   /*			     */
 	ParseOrReturn(sp,&NodeThen(*knp));	   /*			     */
-	Expect(*sp,'}');			   /*			     */
-	Expect(*sp,'{');			   /*			     */
+	Expect(*sp,'}',1);			   /*			     */
+	Expect(*sp,'{',1);			   /*			     */
 	ParseOrReturn(sp,&NodeElse(*knp));	   /*			     */
-	Expect(*sp,'}');			   /*			     */
+	Expect(*sp,'}',1);			   /*			     */
 	knp = & NodeNext(*knp);			   /*			     */
 	cp = *sp;				   /*			     */
 	break;					   /*			     */
@@ -1155,7 +1309,7 @@ static int fmt__parse(sp,knp)			   /*			     */
       case '{':					   /*			     */
 	(*sp)++;				   /*			     */
 	ParseOrReturn(sp,knp);			   /*			     */
-	Expect(*sp,'}');			   /*			     */
+	Expect(*sp,'}',1);			   /*			     */
 	knp = & NodeNext(*knp);			   /*			     */
 	cp = *sp;				   /*			     */
 	break;					   /*			     */
@@ -1212,6 +1366,7 @@ static int eval__fmt(kn,rec)			   /*			     */
   register Record rec;				   /*			     */
 { char	   *s;					   /*			     */
   int	   pos;					   /*			     */
+  char     *trans;				   /*                        */
 						   /*			     */
   while ( kn != (KeyNode)0 )			   /*			     */
   { switch ( NodeType(kn) )			   /*			     */
@@ -1260,43 +1415,72 @@ static int eval__fmt(kn,rec)			   /*			     */
 	{					   /*			     */
 	  DebugPrint1("Field found");	   	   /*			     */
  						   /*                        */
+	  if (NodeType(kn)&NodeMinusMask) 	   /*                        */
+	  { trans = trans_lower; }		   /*                        */
+	  else if (NodeType(kn)&NodePlusMask) 	   /*                        */
+	  { trans = trans_upper; }		   /*                        */
+	  else					   /*                        */
+	  { trans = trans_id; }			   /*                        */
 #define UsePostOr(X) (NodePost(kn) > 0 ? NodePost(kn) : (X))
 #define UsePreOr(X)  (NodePre(kn)  > 0 ? NodePre(kn)  : (X))
-#define UseCase	     (NodeType(kn)&NodeMinusMask ? trans_lower	\
-		      :NodeType(kn)&NodePlusMask ? trans_upper : trans_id)
+#define HasMinus     (NodeType(kn)&NodeMinusMask)
 						   /*			     */
-	  switch ( NodeType(kn)	& 0xff )	   /*			     */
+	  switch ( NodeType(kn)	& 0x1ff )	   /*			     */
 	  {					   /*                        */
-#ifdef NEW
-	    case 'l':				   /*			     */
-	      fmt_count(s,UsePreOr(0));		   /*			     */
+	    case 'p':				   /*			     */
+	      fmt_names(s,UsePreOr(2),UsePostOr(0),trans);/*                 */
 	      break;				   /*			     */
-#endif
 	    case 'n':				   /*			     */
-	      fmt_names(s,UsePreOr(2),UsePostOr(0),UseCase,FALSE);/*	     */
+	      NameStrip(format[0]) = UsePostOr(-1);/*                        */
+	      fmt_names(s,UsePreOr(2),0,trans);    /*                        */
 	      break;				   /*			     */
 	    case 'N':				   /*			     */
-	      fmt_names(s,UsePreOr(2),UsePostOr(0),UseCase,TRUE);/*	     */
+	      NameStrip(format[1]) = UsePostOr(-1);/*                        */
+	      if (NextName(format[1]))		   /*                        */
+	      { NamePre(NextName(format[1])) = NamePreSep;/*                 */
+	      }					   /*                        */
+	      fmt_names(s,UsePreOr(2),1,trans);    /*                        */
 	      break;				   /*			     */
 	    case 'T':				   /*			     */
-	      fmt_title(s,UsePreOr(1),UsePostOr(0),UseCase,TRUE,TitleTitleSep);
+	      fmt_title(s,UsePreOr(1),UsePostOr(0),trans,TRUE,TitleTitleSep);
 	      break;				   /*			     */
 	    case 't':				   /*			     */
-	      fmt_title(s,UsePreOr(1),UsePostOr(0),UseCase,FALSE,TitleTitleSep);
+	      fmt_title(s,UsePreOr(1),UsePostOr(0),trans,FALSE,TitleTitleSep);
 	      break;				   /*			     */
 	    case 'd':				   /*			     */
 	      fmt_digits(s,UsePreOr(0xffff));	   /*			     */
 	      break;				   /*			     */
 	    case 's':				   /*			     */
-	      key_copy(s,UsePreOr(0xffff),UseCase);/*			     */
+	      fmt_string(s,UsePreOr(0xffff),trans);/*			     */
 	      break;				   /*			     */
 	    case 'W':				   /*			     */
-	      fmt_title(s,UsePreOr(1),UsePostOr(0),UseCase,TRUE,"");/*       */
+	      fmt_title(s,UsePreOr(1),UsePostOr(0),trans,TRUE,"");/*         */
 	      break;				   /*			     */
 	    case 'w':				   /*			     */
-	      fmt_title(s,UsePreOr(1),UsePostOr(0),UseCase,FALSE,"");/*      */
+	      fmt_title(s,UsePreOr(1),UsePostOr(0),trans,FALSE,"");/*        */
 	      break;				   /*			     */
 	    default: return 1;			   /*			     */
+	    case 'p' | NodeCountMask:		   /*			     */
+	    case 'n' | NodeCountMask:		   /*			     */
+	    case 'N' | NodeCountMask:		   /*			     */
+	      if (fmt_c_names(s,UsePreOr(0),UsePostOr(0),HasMinus))/*        */
+	      { return 1; }			   /*	                     */
+	      break;				   /*			     */
+	    case 'd' | NodeCountMask:		   /*			     */
+	    case 's' | NodeCountMask:		   /*			     */
+	      if (fmt_c_string(s,UsePreOr(0),UsePostOr(0),HasMinus))/*       */
+	      { return 1; }			   /*	                     */
+	      break;				   /*			     */
+	    case 'T' | NodeCountMask:		   /*			     */
+	    case 'W' | NodeCountMask:		   /*			     */
+	      if (fmt_c_words(s,UsePreOr(0),UsePostOr(0),HasMinus,TRUE))/*   */
+	      { return 1; }			   /*	                     */
+	      break;				   /*			     */
+	    case 't' | NodeCountMask:		   /*			     */
+	    case 'w' | NodeCountMask:		   /*			     */
+	      if (fmt_c_words(s,UsePreOr(0),UsePostOr(0),HasMinus,FALSE))/*  */
+	      { return 1; }			   /*	                     */
+	      break;				   /*			     */
 	  }					   /*			     */
 	}					   /*			     */
 	else { return 1; }			   /*			     */
@@ -1320,7 +1504,7 @@ static void eval__special(kn,rec)		   /*			     */
   register Record  rec;				   /*			     */
 { register char	   *s;				   /*			     */
   int		   missing	= TRUE,		   /*			     */
-		   initialp;			   /*			     */
+		   fmt;			   	   /*			     */
   static char	   *s_author	= NULL;		   /*			     */
   static char	   *s_editor	= NULL;		   /*			     */
   static char	   *s_title	= NULL;		   /*			     */
@@ -1335,15 +1519,16 @@ static void eval__special(kn,rec)		   /*			     */
     s_key	= sym_add("key",-1);		   /*			     */
   }						   /*			     */
 						   /*			     */
-  initialp = (	 NodePre(kn)==KEYSTYLE_LONG	   /*			     */
-	      || NodePre(kn)==KEYSTYLE_LONG_NEED );/*			     */
-						   /*			     */
+  fmt = (   NodePre(kn)==KEYSTYLE_LONG	   	   /*			     */
+	 || NodePre(kn)==KEYSTYLE_LONG_NEED ? 1 : 0 );/*                     */
+  NameStrip(format[fmt]) = UsePostOr(-1);	   /*                        */
+    					   	   /*			     */
   IfGetField(s,s_author)			   /*			     */
-  { fmt_names(s,2,0,trans_lower,initialp);	   /*			     */
+  { fmt_names(s,2,fmt,trans_lower);	   	   /*                        */
     missing = FALSE;				   /*			     */
   }						   /*			     */
   else IfGetField(s,s_editor)			   /*			     */
-  { fmt_names(s,2,0,trans_lower,initialp);	   /*			     */
+  { fmt_names(s,2,fmt,trans_lower);	   	   /*                        */
     missing = FALSE;				   /*			     */
   }						   /*			     */
 						   /*			     */
