@@ -18,6 +18,10 @@
                   kn_pre,
                   kn_post;
    char           *kn_string;
+#ifdef REGEX
+   char           *kn_from,
+		  *kn_to;
+#endif
    struct kEYnODE *kn_next,
                   *kn_then,
                   *kn_else;
@@ -30,11 +34,16 @@
 #define NodeNext(X)   ((X)->kn_next)
 #define NodeThen(X)   ((X)->kn_then)
 #define NodeElse(X)   ((X)->kn_else)
+#ifdef REGEX
+#define NodeFrom(X)   ((X)->kn_from)
+#define NodeTo(X)     ((X)->kn_to)
+#endif
 
-#define NodePlusMask   256
-#define NodeMinusMask  512
-#define NodeSTRING    1024
-#define NodeTEST      1025
-#define NodeTESTneg   1026
-#define NodeOR        1027
-#define NodeSPECIAL   1028
+#define NodeCountMask 0x100
+#define NodePlusMask  0x200
+#define NodeMinusMask 0x400
+#define NodeSTRING    2048
+#define NodeTEST      2049
+#define NodeTESTneg   2050
+#define NodeOR        2051
+#define NodeSPECIAL   2052
