@@ -127,7 +127,7 @@ void usage(full)				   /*			     */
   int           full;				   /*                        */
 { register char **cpp;				   /*			     */
 						   /*			     */
-  ErrPrintF("BibTool Vers. %s (C) 1995 Gerd Neugebauer\n\n",version);/*	     */
+  ErrPrintF("BibTool Vers. %s (C) 1996 Gerd Neugebauer\n\n",version);/*	     */
 						   /*			     */
   if ( full )					   /*                        */
   { for ( cpp=use; *cpp; cpp++ )		   /*			     */
@@ -295,6 +295,7 @@ int main(argc,argv)				   /*			     */
   init_symbols();				   /*			     */
   init_entries();				   /*			     */
   init_read();					   /*			     */
+  init_key(0);					   /*			     */
   set_rsc_path(rsc_v_rsc);			   /*                        */
 						   /*			     */
   for ( i=1; i<argc; i++ )			   /*			     */
@@ -332,11 +333,13 @@ int main(argc,argv)				   /*			     */
 	case 'v': Toggle(rsc_verbose);	    break; /* verbose		     */
 	case 'V': usage(FALSE);	            break; /* version		     */
 	case 'x':				   /* extract		     */
+	  rsc_all_macs = FALSE;			   /*                        */
 	  if ( ++i < argc )			   /*			     */
 	  { read_aux(argv[i],*++ap=='v'); }	   /*			     */
 	  else		    { NoSFileWarning; }	   /*			     */
 	  break;				   /*			     */
 	case 'X':				   /* extract pattern	     */
+	  rsc_all_macs = FALSE;			   /*                        */
 	  if ( ++i < argc ) { save_regex(argv[i]); }/*			     */
 	  else		    { MissingPattern; }	   /*			     */
 	  break;				   /*			     */
@@ -374,6 +377,7 @@ int main(argc,argv)				   /*			     */
 						   /*			     */
   init_macros();				   /*			     */
   init_read();					   /*			     */
+  init_key(1);					   /*			     */
 						   /*			     */
   for ( i=0; i<input_file_ptr; i++ )		   /* For all input files    */
   {						   /*			     */
